class PushsController < ApplicationController
  include ActionController::HttpAuthentication::Token
  # skip_before_action :verify_authenticity_token

  def token
    param! :app_name, String, required: true
    param! :app_id, String, required: true
    param! :app_secret, String, required: true

    app_setting = Settings.send(params[:app_name].to_sym) rescue {}
    if app_setting['app_id'] == params[:app_id] && app_setting['app_secret'] == params[:app_secret]
      render json: { code: 0, message: 'success', token: generate_token(params[:app_name]), ttl: token_ttl(params[:app_name])}
    else
      render json: { code: -1, message: '应用不存在或者app_id/app_secret不匹配'}
    end
  end
  
  def single
    param! :app_name, String, required: true, in: %w(crm)
    # param! :device_ids_opts, String, required: true
    param! :message, String, required: true
    param! :igetui_opts, String
    param! :app_type, String
    param! :sync_push, :boolean
    param! :user_id, String, required: true
    param! :db_role, String, in: %w(dev test staging production)


    igetui_opts = JSON.parse(params[:igetui_opts]) rescue {}
    message = JSON.parse(params[:message]) rescue {}

    db_role = fetch_db_role
    user_device_exist = false
    ActiveRecord::Base.connected_to(role: db_role) do
      user_device_exist = UserDevice.where(user_id: params[:user_id]).exists?
      byebug
      stale?(last_modified: UserDevice.where(user_id: params[:user_id]).last.updated_at)
    end

    unless user_device_exist
      return render json: { code: -1, message: 'user_device找不到，user_id错误'}
    end

    token_array = token_and_options(request)
    if token_array.blank?
      return render json: { code: -1, message: 'token错误'}
    end
    if validate_token(params[:app_name], token_array.first)
      push_sync(params[:sync_push], {db_role: db_role, user_ids: Array(params[:user_id]), message: message, app_type: params[:app_type], igetui_opts: igetui_opts.merge(pusher_type: 'push_message_to_single')})
     
      
      render json: { code: 0, message: 'success', describe: '异步任务正在处理'}
    else
      render json: { code: -1, message: 'token 错误/过期'}
    end
    
  end
  
  def batch
    param! :app_name, String, required: true, in: %w(crm) 
    # param! :device_ids_opts, String, required: true
    param! :message, String, required: true
    param! :igetui_opts, String
    param! :app_type, String
    param! :sync_push, String
    param! :user_ids, Array, required: true
    param! :db_role, String, in: %w(dev test staging production)

    igetui_opts = JSON.parse(params[:igetui_opts]) rescue {}
    message = JSON.parse(params[:message]) rescue {}
    user_ids = params[:user_ids] rescue []

    db_role = fetch_db_role
    user_device_exist = false
    ActiveRecord::Base.connected_to(role: db_role) do
      user_device_exist = UserDevice.where(user_id: user_ids).exists?
    end
    unless user_device_exist
      return render json: { code: -1, message: 'user_device找不到,user_ids错误'}
    end

    token_array = token_and_options(request)
    if token_array.blank?
      return render json: { code: -1, message: 'token错误'}
    end

    if validate_token(params[:app_name], token_array.first)
      push_sync(params[:sync_push], {db_role: db_role, user_ids: user_ids, message: message, app_type: params[:app_type], igetui_opts: igetui_opts.merge(pusher_type: 'push_message_to_list')})

      render json: { code: 0, message: 'success', describe: '异步任务正在处理'}
    else
      render json: { code: -1, message: 'token 错误/过期'}
    end
    
  end
  
  
  private

  def push_sync(sync_push, opts={})
    if sync_push
      timeout_seconds = $redis.get('push_timeout').to_i
      if timeout_seconds > 0
        Timeout.timeout(timeout_seconds) {
          ::PushWorker.new.perform(opts)
        }
      else
        ::PushWorker.new.perform(opts)
      end
    else
      ::PushWorker.perform_async(opts)
    end
  end
  
  def fetch_db_role
    return params[:db_role].to_sym if params[:db_role].present?
    
    case request.host
      when 'lxcrm-dev.weiwenjia.com', 'ik-dev.ikcrm.com'
        :dev
      when 'lxcrm-test.weiwenjia.com', 'ik-test.weiwenjia.com'
        :test
      when 'lxcrm-staging.weiwenjia.com', 'ik-staging.weiwenjia.com'
        :staging
      else
        :production
    end
  end
end
